/* ===========================================================================

    keypad.c  - KEYPAD routines for Axiom CMLU-9S12C32 on Port P

    Version:        1.0
    Author:         Axiom Manufacturing (www.axman.com)
    Compiler:       CodeWarrior for S12(X) 5.1

    This is freeware - use as you like

==============================================================================
------------------------------------------------------------------------------
*/

#include "derivative.h"			// derivative-specific definitions
#include "keypad.h" 

// Constant Definitions
// --------------------
const char keytable[] = ".123A456B789C*0#D";

// RAM Variables
// -------------
unsigned char Newkey, Oldkey;


/*----------------------------------------------------------------------------
InitKeyPad
----------
 Initialize port p (PTP) KEYPAD port used. Scan function, 
 no interrupt

     Port P 0-3   output
            4-7 	input with pull-down enabled 
 ----------------------------------------------------------------------------*/
void InitKeypad(void){
    PTP = 0x00;			// init PORT P output
    DDRP = 0x0F;			// set Port P direction (0=input, 1=output)
    PPSP = 0xF0;			// set for pull downs
    PERP = 0xF0;			// enable pull downs
    PIFP = 0xF0;			// clear pending flags
//    PIEP = 0xF0;			// enable interrupts PTP[7:4]
    
    Newkey=0; Oldkey=0;	// reset keypad status variables
}


/*----------------------------------------------------------------------------
kbhit
-----
 Does a KEYPAD scan.

 returns 0 if NO key is pressed
 otherwise returns the representation of the key pressed from the included table

 ----------------------------------------------------------------------------*/
char kbhit(void){
    unsigned char column, keycount, kstat, delaytime;

    //PTP = 0x0F;					// set all KEYPAD columns
    kstat = PTP;					// get KEYPAD data
    kstat &= 0xF0;				// mask rows
    if(kstat != 0){				// if key pressed

        for(column = 0x01, keycount=0; column < 0x10; column <<= 1, keycount++){
            PTP = 0;				// clear all KEYPAD columns
            PTP = column;		// set this KEYPAD column
            for(delaytime=0x20; delaytime > 0; delaytime--) ;   // wait for lines to settle
            kstat = PTP;		// get KEYPAD data
            kstat &= 0xF0;		// mask rows
            if(kstat != 0){	// if key pressed on this column
                kstat >>= 4;
                if(kstat == 0x04)   kstat = 3;
                if(kstat == 0x08)   kstat = 4;
                return(keytable[(keycount*4) + kstat]);
            }
        }
    }
    return 0;   // return no key pressed
}

/*----------------------------------------------------------------------------
getkeypad
---------
 Returns a key pressed on the keypad.  First waits for any current key pressed
 to be released, then does not return until a new key is debounced.

 Returns the representation of the key pressed from the array keytable[]
 ----------------------------------------------------------------------------*/
char getkeypad(void){
    char debcount;  // debounce counter

    do{
        Newkey = kbhit();   // get key pressed status
    }while(Newkey == Oldkey);   // while same key pressed as last scan

    for(debcount=0; debcount < KEY_DEBTIME; debcount++){
        Newkey = kbhit();
        if(Newkey == 0 || Newkey != Oldkey){    // if key not still pressed
            debcount=0; // clear debounce counter
        }
        Oldkey = Newkey;    // save for next scan
    }
    return(Newkey);
}

